@@ -1,30 +1,30 @@
 use ruma::events::*;
 pub use ruma::{
-    api::client::r0::sync::sync_events::*, identifiers::RoomName, presence::PresenceState,
-    serde::Raw, DeviceKeyAlgorithm, EventId, RoomAliasId, RoomId, UInt, UserId,
+    api::client::sync::sync_events::v3::Response, identifiers::RoomName, EventId, RoomAliasId,
+    RoomId, ServerName, UserId,
 };
 use serde::{Deserialize, Serialize};
-use serde_json::{json, Value};
+//use serde_json::{json, Value};
 use std::collections::BTreeMap;
 use std::time::Duration;
-use ureq::{Agent, AgentBuilder, OrAnyStatus};
-use uuid::Uuid;
-const BASE: &str = "/_matrix/client/v3/";
+//use ureq::{Agent, AgentBuilder, OrAnyStatus};
+//use uuid::Uuid;
+const TIMEOUT: Duration = Duration::from_secs(3);
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
 pub struct StorageData {
     pub auth_token: String,
-    pub room_to_aliases: BTreeMap<RoomId, Vec<RoomAliasId>>,
-    pub alias_to_room: BTreeMap<RoomAliasId, RoomId>,
-    pub joined_rooms: Vec<RoomId>,
-    pub room_id_to_cananoical_alias: BTreeMap<RoomId, RoomAliasId>,
-    pub room_to_name: BTreeMap<RoomId, Box<RoomName>>,
-    pub room_to_direct_name: BTreeMap<RoomId, String>,
-    pub recent_event_ids: Vec<EventId>,
-    pub last_sent_messages: Vec<(RoomId, String)>,
+    pub room_to_aliases: BTreeMap<Box<RoomId>, Vec<Box<RoomAliasId>>>,
+    pub alias_to_room: BTreeMap<Box<RoomAliasId>, Box<RoomId>>,
+    pub joined_rooms: Vec<Box<RoomId>>,
+    pub room_id_to_cananoical_alias: BTreeMap<Box<RoomId>, Box<RoomAliasId>>,
+    pub room_to_name: BTreeMap<Box<RoomId>, Box<RoomName>>,
+    pub room_to_direct_name: BTreeMap<Box<RoomId>, String>,
+    pub recent_event_ids: Vec<Box<EventId>>,
+    pub last_sent_messages: Vec<(Box<RoomId>, String)>,
 }
-
+/*
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
-pub struct EventResponse {
+pub struct Response {
     /// The batch token to supply in the `since` param of the next `/sync` request.
     pub next_batch: String,
 
@@ -62,30 +62,65 @@ pub struct EventResponse {
     /// fallback keys.
     #[serde(rename = "org.matrix.msc2732.device_unused_fallback_key_types")]
     pub device_unused_fallback_key_types: Option<Vec<DeviceKeyAlgorithm>>,
-}
+}*/
+#[allow(dead_code)]
 #[derive(Debug, Clone)]
 pub struct Client {
-    agent: Agent,
-    pub me: UserId,
+    client: ruma::Client<ruma::client::http_client::Reqwest>,
+    pub me: Box<UserId>,
     since: Option<String>,
     host: String,
     pub data: StorageData,
     room_list_updated: bool,
 }
-
+pub enum Authentication {
+    Password(String),
+    AuthToken(String),
+}
 impl Client {
-    pub fn new(
+    pub async fn new(
         host: impl Into<String>,
         username: impl Into<String>,
         server_name: impl Into<String>,
+        authentication: Authentication,
     ) -> Self {
-        Self {
-            host: host.into().trim_end_matches("/").to_string(),
-            me: UserId::try_from(format!("@{}:{}", username.into(), server_name.into()).as_str())
+        let good_host = host.into().trim_end_matches("/").to_string();
+        let me = UserId::parse_with_server_name(
+            username.into(),
+            &ruma::ServerName::parse(server_name.into()).unwrap(),
+        )
+        .unwrap();
+        let client = match authentication {
+            Authentication::Password(x) => {
+                let c = ruma::Client::builder()
+                    .homeserver_url(good_host.clone())
+                    .build()
+                    .await
+                    .unwrap();
+                c.log_in(me.clone().as_str(), x.as_str(), None, None)
+                    .await
+                    .unwrap();
+                c
+            }
+            Authentication::AuthToken(x) => ruma::Client::builder()
+                .homeserver_url(good_host.clone())
+                .access_token(Some(x))
+                .build()
+                .await
                 .unwrap(),
-            agent: AgentBuilder::new()
-                .timeout_read(Duration::from_secs(3))
-                .build(),
+        };
+        Self {
+            client: client,
+            host: good_host, //host.into().trim_end_matches("/").to_string(),
+            me: me,          /*UserId::parse_with_server_name(
+                                 username.into(),
+                                 &*ruma::ServerName::parse(server_name.into()).unwrap(),
+                             )
+                             .unwrap(),
+                             agent: AgentBuilder::new()
+                                 .timeout_read(Duration::from_secs(3))
+                                 .build(),
+                             */
             since: None,
             data: StorageData::default(),
             room_list_updated: false,
@@ -98,6 +133,7 @@ impl Client {
         }
         false
     }
+    /*
     pub fn set_auth_token(&mut self, auth_token: String) {
         let should_req_well_known = match self.host.split(":").last().unwrap().parse::<usize>() {
             Ok(port) => {
@@ -167,9 +203,11 @@ impl Client {
         self.me = UserId::try_from(val["user_id"].as_str().unwrap()).unwrap();
         Ok(())
     }
-    pub fn get_me(&self) -> UserId {
+    */
+    pub fn get_me(&self) -> Box<UserId> {
         self.me.clone()
     }
+    /*
     fn post(
         &self,
         url_ext: impl Into<String>,
@@ -295,7 +333,46 @@ impl Client {
             .into_json()
             .unwrap()
     }
-    pub fn sync(&mut self) -> Result<EventResponse, Value> {
+    */
+    pub async fn join_room_id(
+        &self,
+        room_id: Box<ruma::RoomId>,
+    ) -> ruma::api::client::membership::join_room_by_id::v3::Response {
+        self.client
+            .send_request(
+                ruma::api::client::membership::join_room_by_id::v3::Request::new(&room_id),
+            )
+            .await
+            .unwrap()
+    }
+    pub async fn sync(&mut self) -> Result<Response, ()> {
+        //let since: Option<&'a str>;
+        //since = Some(self.since.unwrap().as_str())
+        /*
+        if let Some(start) = self.since.clone() {
+            since = Some(&start);
+        } else {
+            since = None;
+        }
+        */
+        let mut req = ruma::api::client::sync::sync_events::v3::Request::new();
+        req.filter = None;
+        req.since = match self.since {
+            Some(ref x) => Some(x),
+            None => None,
+        };
+        req.full_state = self.since.clone().is_none();
+        //set_presence: &PresenceState::Online,
+        req.timeout = Some(TIMEOUT);
+        if let Ok(res) = self.client.send_request(req).await {
+            self.since = Some(res.clone().next_batch);
+            return Ok(res);
+        } else {
+            return Err(());
+        }
+    }
+    /*
+    pub fn sync(&mut self) -> Result<Response, Value> {
         let req: ureq::Response;
         if let Some(start) = self.since.clone() {
             req = self
@@ -331,15 +408,21 @@ impl Client {
         if val.clone().as_object().unwrap().contains_key("error") {
             Err(val)
         } else {
-            let x: EventResponse =
-                serde_json::from_str::<Raw<EventResponse>>(val.to_string().as_str())
-                    .unwrap()
-                    .deserialize()
-                    .unwrap();
+            let x: Response = serde_json::from_str::<Raw<Response>>(val.to_string().as_str())
+                .unwrap()
+                .deserialize()
+                .unwrap();
             self.since = Some(x.clone().next_batch);
             Ok(x)
         }
     }
+    */
+    pub async fn join_with_alias(&self, alias: impl Into<String>) {
+        let room_alias = alias.into();
+        self.join_room_id(self.room_id_from_alias(room_alias).await)
+            .await;
+    }
+    /*
     pub fn join_with_alias(&self, alias: impl Into<String>) {
         let room_alias = alias.into();
         let this_room_id_val: Value = self
@@ -349,7 +432,7 @@ impl Client {
                     urlencoding::encode(room_alias.as_str())
                         .to_string()
                         .as_str()
-                ),
+                )
                 None,
             )
             .unwrap()
@@ -359,7 +442,7 @@ impl Client {
             ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap();
         self.join_room_id(room_id);
     }
-    pub fn leave_room_id(&mut self, room_id: RoomId) -> Value {
+    pub fn leave_room_id(&mut self, room_id: Box<RoomId>) -> Value {
         let rid = room_id.to_string();
         let room = urlencoding::encode(rid.as_str());
         let content = json!({});
@@ -372,7 +455,24 @@ impl Client {
         self.room_list_updated = true;
         output
     }
-    pub fn room_id_from_alias(&self, alias: impl Into<String>) -> RoomId {
+    */
+    pub async fn leave_room_id(
+        &mut self,
+        room_id: Box<RoomId>,
+    ) -> ruma::api::client::membership::leave_room::v3::Response {
+        let resp = self
+            .client
+            .send_request(ruma::api::client::membership::leave_room::v3::Request::new(
+                &room_id,
+            ))
+            .await
+            .unwrap();
+        self.data.joined_rooms.retain(|x| x != &room_id);
+        self.room_list_updated = true;
+        resp
+    }
+    /*
+    pub fn room_id_from_alias(&self, alias: impl Into<String>) -> Box<RoomId> {
         let room_alias = alias.into();
         let this_room_id_val: Value = self
             .get(
@@ -389,7 +489,20 @@ impl Client {
             .unwrap();
         ruma::RoomId::try_from(this_room_id_val["room_id"].as_str().unwrap()).unwrap()
     }
-    pub fn send_message(&mut self, room_id: RoomId, message: impl Into<String>) {
+    */
+    pub async fn room_id_from_alias(&self, alias: impl Into<String>) -> Box<RoomId> {
+        let room_alias_string = alias.into();
+        let room_alias = RoomAliasId::parse(room_alias_string.as_str()).unwrap();
+        self.client
+            .send_request(ruma::api::client::alias::get_alias::v3::Request::new(
+                &room_alias,
+            ))
+            .await
+            .unwrap()
+            .room_id
+    }
+    /*
+    pub fn send_message(&mut self, room_id: Box<RoomId>, message: impl Into<String>) {
         let msg = message.into();
         let txid = Uuid::new_v4()
             .to_simple()
@@ -403,8 +516,29 @@ impl Client {
         .unwrap();
         self.data.last_sent_messages.push((room_id.clone(), msg));
     }
-
-    pub fn extract_data(&mut self, sync_results: EventResponse) -> Vec<MessageInfo> {
+    */
+    pub async fn send_message(
+        &mut self,
+        room_id: Box<RoomId>,
+        message: impl Into<String>,
+    ) -> ruma::api::client::message::send_message_event::v3::Response {
+        let msg = message.into();
+        let resp = self
+            .client
+            .send_request(
+                ruma::api::client::message::send_message_event::v3::Request::new(
+                    &room_id,
+                    &ruma::TransactionId::new(),
+                    &ruma::events::room::message::RoomMessageEventContent::text_plain(msg.clone()),
+                )
+                .unwrap(),
+            )
+            .await
+            .unwrap();
+        self.data.last_sent_messages.push((room_id.clone(), msg));
+        resp
+    }
+    pub fn extract_data(&mut self, sync_results: Response) -> Vec<MessageInfo> {
         let mut messages: Vec<MessageInfo> = Vec::new();
         let x = sync_results.rooms.join;
         for (room_id, joined_room) in x.iter() {
@@ -481,12 +615,12 @@ impl Client {
                 if let Ok(ruma::events::AnySyncRoomEvent::Message(message_event)) =
                     m.deserialize_as()
                 {
-                    let msg_event_id = message_event.event_id();
-                    if self.data.recent_event_ids.contains(msg_event_id) {
+                    let msg_event_id = Box::new(message_event.event_id().to_owned());
+                    if self.data.recent_event_ids.contains(&msg_event_id) {
                         return None;
                     }
-                    if !self.data.recent_event_ids.contains(msg_event_id) {
-                        self.data.recent_event_ids.push(msg_event_id.clone());
+                    if !self.data.recent_event_ids.contains(&msg_event_id) {
+                        self.data.recent_event_ids.push(*msg_event_id);
                     }
                     if self.data.recent_event_ids.len() > 500 {
                         self.data.recent_event_ids.remove(0);
@@ -537,8 +671,8 @@ impl Client {
 }
 
 pub struct MessageInfo {
-    pub room_id: RoomId,
-    pub sender: UserId,
+    pub room_id: Box<RoomId>,
+    pub sender: Box<UserId>,
     pub message: String,
     pub is_emote: bool,
 }
